@@ -27,6 +27,7 @@
 
 #include <libedata-book/libedata-book.h>
 
+#include "server/e-oauth2-service-office365.h"
 #include "server/e-source-ews-folder.h"
 
 #include "e-book-backend-ews.h"
@@ -80,6 +81,7 @@ e_module_load (GTypeModule *type_module)
 	e_module = E_MODULE (type_module);
 
 	e_source_ews_folder_type_register (type_module);
+	e_oauth2_service_office365_type_register (type_module);
 
 	e_book_backend_ews_factory_register_type (type_module);
 }
