@@ -61,6 +61,8 @@ static CamelProviderConfEntry ews_conf_entries[] = {
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "stay-synchronized", NULL,
 	  N_("Synchroni_ze remote mail locally in all folders"), "0" },
 	{ CAMEL_PROVIDER_CONF_PLACEHOLDER, "ews-limit-by-age-placeholder", NULL },
+	{ CAMEL_PROVIDER_CONF_CHECKBOX, "show-public-folders", NULL,
+	  N_("Show _Public Folders beside user folders"), "0" },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 
 	{ CAMEL_PROVIDER_CONF_SECTION_START, "connection", NULL, N_("Connection") },
@@ -68,6 +70,8 @@ static CamelProviderConfEntry ews_conf_entries[] = {
 	  /* Translators: '%s' is preplaced with a widget, where "
 	   * user can select how long the timeout should be. */
 	  N_("Connection _timeout (in seconds) %s"), "0:1:0:32768" },
+	{ CAMEL_PROVIDER_CONF_CHECKSPIN, "concurrent-connections", NULL,
+	  N_("Numbe_r of concurrent connections to use"), "y:1:1:7" },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "override-user-agent", NULL,
 	  N_("Override _User-Agent header value"), "0" },
 	{ CAMEL_PROVIDER_CONF_ENTRY, "user-agent", "override-user-agent", "" },
