@@ -251,7 +251,7 @@ e_ews_check_foreign_folder_data_free (gpointer ptr)
 	if (cffd->folder)
 		g_object_unref (cffd->folder);
 
-	g_free (cffd);
+	g_slice_free (struct EEwsCheckForeignFolderData, cffd);
 }
 
 static void
@@ -393,11 +393,12 @@ check_foreign_folder_thread (GObject *with_object,
 	} else {
 		fid.id = (gchar *) (cffd->use_foldername ? cffd->use_foldername : cffd->orig_foldername);
 		fid.change_key = NULL;
-		fid.is_distinguished_id = cffd->use_foldername != NULL;
+		fid.is_distinguished_id = cffd->use_foldername != NULL || (cffd->orig_foldername && strlen (cffd->orig_foldername) < 40);
 
 		if (!e_ews_connection_get_folder_info_sync (conn, G_PRIORITY_DEFAULT,
 			cffd->email, &fid, &folder, cancellable, &local_error)) {
-			if (g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_ITEMNOTFOUND) ||
+			if (!local_error ||
+			    g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_ITEMNOTFOUND) ||
 			    g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_FOLDERNOTFOUND)) {
 				g_clear_error (&local_error);
 				local_error = g_error_new (
@@ -596,7 +597,7 @@ subscribe_foreign_response_cb (GObject *dialog,
 		show_foldername = g_strdup_printf ("%.10s…", orig_foldername);
 	}
 
-	cffd = g_new0 (struct EEwsCheckForeignFolderData, 1);
+	cffd = g_slice_new0 (struct EEwsCheckForeignFolderData);
 	cffd->dialog = GTK_WIDGET (dialog);
 	cffd->email = g_strdup (username ? username : "");
 	cffd->direct_email = g_strdup (g_object_get_data (G_OBJECT (entry), STR_EWS_DIRECT_EMAIL));
