@@ -34,12 +34,14 @@ struct _CamelEwsSettingsPrivate {
 	gboolean filter_junk_inbox;
 	gboolean oab_offline;
 	gboolean use_impersonation;
+	gboolean show_public_folders;
 	gchar *email;
 	gchar *gal_uid;
 	gchar *hosturl;
 	gchar *oaburl;
 	gchar *oal_selected;
 	guint timeout;
+	guint concurrent_connections;
 	gchar *impersonate_user;
 	gboolean override_user_agent;
 	gchar *user_agent;
@@ -74,7 +76,9 @@ enum {
 	PROP_OVERRIDE_OAUTH2,
 	PROP_OAUTH2_TENANT,
 	PROP_OAUTH2_CLIENT_ID,
-	PROP_OAUTH2_REDIRECT_URI
+	PROP_OAUTH2_REDIRECT_URI,
+	PROP_SHOW_PUBLIC_FOLDERS,
+	PROP_CONCURRENT_CONNECTIONS
 };
 
 G_DEFINE_TYPE_WITH_CODE (
@@ -263,6 +267,18 @@ ews_settings_set_property (GObject *object,
 				CAMEL_EWS_SETTINGS (object),
 				g_value_get_string (value));
 			return;
+
+		case PROP_SHOW_PUBLIC_FOLDERS:
+			camel_ews_settings_set_show_public_folders (
+				CAMEL_EWS_SETTINGS (object),
+				g_value_get_boolean (value));
+			return;
+
+		case PROP_CONCURRENT_CONNECTIONS:
+			camel_ews_settings_set_concurrent_connections (
+				CAMEL_EWS_SETTINGS (object),
+				g_value_get_uint (value));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -442,6 +458,20 @@ ews_settings_get_property (GObject *object,
 				camel_ews_settings_dup_oauth2_redirect_uri (
 				CAMEL_EWS_SETTINGS (object)));
 			return;
+
+		case PROP_SHOW_PUBLIC_FOLDERS:
+			g_value_set_boolean (
+				value,
+				camel_ews_settings_get_show_public_folders (
+				CAMEL_EWS_SETTINGS (object)));
+			return;
+
+		case PROP_CONCURRENT_CONNECTIONS:
+			g_value_set_uint (
+				value,
+				camel_ews_settings_get_concurrent_connections (
+				CAMEL_EWS_SETTINGS (object)));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -740,6 +770,33 @@ camel_ews_settings_class_init (CamelEwsSettingsClass *class)
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT |
 			G_PARAM_STATIC_STRINGS));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_SHOW_PUBLIC_FOLDERS,
+		g_param_spec_boolean (
+			"show-public-folders",
+			"Show Public Folders",
+			NULL,
+			FALSE,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_STATIC_STRINGS));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_CONCURRENT_CONNECTIONS,
+		g_param_spec_uint (
+			"concurrent-connections",
+			"Concurrent Connections",
+			"Number of concurrent connections to use",
+			MIN_CONCURRENT_CONNECTIONS,
+			MAX_CONCURRENT_CONNECTIONS,
+			1,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_EXPLICIT_NOTIFY |
+			G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -1543,3 +1600,52 @@ camel_ews_settings_set_oauth2_redirect_uri (CamelEwsSettings *settings,
 
 	g_object_notify (G_OBJECT (settings), "oauth2-redirect-uri");
 }
+
+gboolean
+camel_ews_settings_get_show_public_folders (CamelEwsSettings *settings)
+{
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), FALSE);
+
+	return settings->priv->show_public_folders;
+}
+
+void
+camel_ews_settings_set_show_public_folders (CamelEwsSettings *settings,
+					    gboolean show_public_folders)
+{
+	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
+
+	if ((settings->priv->show_public_folders ? 1 : 0) == (show_public_folders ? 1 : 0))
+		return;
+
+	settings->priv->show_public_folders = show_public_folders;
+
+	g_object_notify (G_OBJECT (settings), "show-public-folders");
+}
+
+guint
+camel_ews_settings_get_concurrent_connections (CamelEwsSettings *settings)
+{
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), 1);
+
+	return settings->priv->concurrent_connections;
+}
+
+void
+camel_ews_settings_set_concurrent_connections (CamelEwsSettings *settings,
+					       guint concurrent_connections)
+{
+	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
+
+	concurrent_connections = CLAMP (
+		concurrent_connections,
+		MIN_CONCURRENT_CONNECTIONS,
+		MAX_CONCURRENT_CONNECTIONS);
+
+	if (settings->priv->concurrent_connections == concurrent_connections)
+		return;
+
+	settings->priv->concurrent_connections = concurrent_connections;
+
+	g_object_notify (G_OBJECT (settings), "concurrent-connections");
+}
