@@ -46,6 +46,7 @@ struct _EEwsFolderPrivate {
 	guint64 size;
 	gboolean foreign;
 	gchar *foreign_mail;
+	gboolean is_public;
 };
 
 static void
@@ -108,6 +109,7 @@ e_ews_folder_init (EEwsFolder *folder)
 	folder->priv->error = NULL;
 	folder->priv->folder_type = E_EWS_FOLDER_TYPE_UNKNOWN;
 	folder->priv->foreign = FALSE;
+	folder->priv->is_public = FALSE;
 }
 
 static gboolean
@@ -119,7 +121,8 @@ e_ews_folder_set_from_soap_parameter (EEwsFolder *folder,
 
 	g_return_val_if_fail (param != NULL, FALSE);
 
-	if (g_strcmp0 (e_soap_parameter_get_name (param), "Folder") == 0) {
+	if (g_strcmp0 (e_soap_parameter_get_name (param), "Folder") == 0 ||
+	    g_strcmp0 (e_soap_parameter_get_name (param), "SearchFolder") == 0 ) {
 		node = param;
 		priv->folder_type = E_EWS_FOLDER_TYPE_MAILBOX;
 	} else if (g_strcmp0 (e_soap_parameter_get_name (param), "CalendarFolder") == 0) {
@@ -131,7 +134,8 @@ e_ews_folder_set_from_soap_parameter (EEwsFolder *folder,
 	} else if (g_strcmp0 (e_soap_parameter_get_name (param), "TasksFolder") == 0) {
 		node = param;
 		priv->folder_type = E_EWS_FOLDER_TYPE_TASKS;
-	} else if ((node = e_soap_parameter_get_first_child_by_name (param, "Folder")))
+	} else if ((node = e_soap_parameter_get_first_child_by_name (param, "Folder")) ||
+		   (node = e_soap_parameter_get_first_child_by_name (param, "SearchFolder")))
 		priv->folder_type = E_EWS_FOLDER_TYPE_MAILBOX;
 	else if ((node = e_soap_parameter_get_first_child_by_name (param, "CalendarFolder")))
 		priv->folder_type = E_EWS_FOLDER_TYPE_CALENDAR;
@@ -554,6 +558,23 @@ e_ews_folder_set_foreign_mail (EEwsFolder *folder,
 	folder->priv->foreign_mail = g_strdup (foreign_mail);
 }
 
+gboolean
+e_ews_folder_get_public (const EEwsFolder *folder)
+{
+	g_return_val_if_fail (E_IS_EWS_FOLDER (folder), FALSE);
+
+	return folder->priv->is_public;
+}
+
+void
+e_ews_folder_set_public (EEwsFolder *folder,
+			 gboolean is_public)
+{
+	g_return_if_fail (E_IS_EWS_FOLDER (folder));
+
+	folder->priv->is_public = is_public;
+}
+
 /* escapes backslashes with \5C and forward slashes with \2F */
 gchar *
 e_ews_folder_utils_escape_name (const gchar *folder_name)
